#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import socket
from pathlib import Path

from p1st.common import Message1, Message2, get_inform1_path, get_inform2_path, get_chunk_path
from p1st.exec_produce_chunks import execute_produce_chunks
from p1st.unix_sock_input import accept_loop_until_message


def main():
    args = parse_args()
    #
    chunk_size = args.chunk_size
    compressed_data: bool = args.compressed_data
    child: Path = args.child
    parent: Path | None = args.parent
    source_chunk_dir: Path = args.chunk_dir

    source_chunk_dir.mkdir(parents=True, exist_ok=True)

    command_parts = (
        ['btrfs', 'send'],
        ['-p', str(parent)] if parent else [],
        ['--compress-data'] if compressed_data else [],
        [str(child)]
    )
    command = [x for xs in command_parts for x in xs]

    def get_source_chunk_path(chunk_no: int) -> Path:
        return get_chunk_path(source_chunk_dir, child, chunk_no)

    def handle_chunk(chunk_no: int, last_chunk: bool):
        source_chunk_path = get_source_chunk_path(chunk_no)
        print(f'Handling chunk {source_chunk_path}')

        # Create in_socket
        in_socket: Path = get_inform2_path(source_chunk_dir, child)
        in_sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        in_sock.bind(str(in_socket))
        in_sock.listen(1)

        # Inform receiving side, that the next chunk can be transferred.
        inform_path = get_inform1_path(source_chunk_path)
        inform_path.write_text(Message1.EOF.value if last_chunk else Message1.OK.value)

        # Read from in_socket
        # -> Receiving side informs us after the chunk has been transferred.
        _command = accept_loop_until_message(in_sock, [Message2.OK_BINARY.value])

        # Delete local chunk after it has been transferred.
        source_chunk_path.unlink(missing_ok=False)

        # Close in_socket
        in_sock.close()
        in_socket.unlink(missing_ok=False)

    returncode = execute_produce_chunks(
        command=command,
        get_chunk_path=get_source_chunk_path,
        handle_chunk=handle_chunk,
        chunk_size=chunk_size,
    )
    exit(returncode)


def parse_args():
    parser = argparse.ArgumentParser(prog='btrfs-send-chunks')

    parser.add_argument('-p',
                        help='Path to parent subvolume; forwarded to btrfs-send.',
                        dest='parent',
                        default=None,
                        type=Path,
                        metavar='PARENT_SUBVOLUME'
                        )

    parser.add_argument('--compressed-data',
                        help='Forwarded to btrfs-send.',
                        dest='compressed_data',
                        action='store_true',
                        default=False,
                        )

    parser.add_argument('--chunk-size',
                        help='Size in bytes; defaults to 128 MB.',
                        dest='chunk_size',
                        type=int,
                        default=128 * 1024 * 1024,
                        )

    parser.add_argument('--chunk-dir',
                        help='Chunks are saved in this directory. '
                             'Defaults to parent directory of CHILD.',
                        dest='chunk_dir',
                        type=Path,
                        metavar='CHUNK_DIR',
                        default=None,
                        )

    parser.add_argument('child',
                        help='Path to child subvolume. Forwarded to btrfs-send.',
                        type=Path,
                        metavar='CHILD_SUBVOLUME'
                        )

    args = parser.parse_args()

    # Make all paths absolute. Set default values.
    args.child = args.child.absolute()
    #
    if args.chunk_dir:
        args.chunk_dir = args.chunk_dir.absolute()
    else:  # Default value
        args.chunk_dir = args.child.absolute().parent
    #
    if args.parent:
        args.parent = args.parent.absolute()

    return args


if __name__ == '__main__':
    main()
